@@ -15,20 +15,15 @@ details on the PSF (Python Software Foundation) standard license, see:
 
 http://www.python.org/2.2.3/license.html
 
-$Id: Debugger.py 2203 2007-04-04 05:08:36Z fperez $"""
+$Id: Debugger.py 2913 2007-12-31 12:42:14Z vivainio $"""
 
 #*****************************************************************************
 #
-# Since this file is essentially a modified copy of the pdb module which is
-# part of the standard Python distribution, I assume that the proper procedure
-# is to maintain its copyright as belonging to the Python Software Foundation
-# (in addition to my own, for all new code).
+#       This file is licensed under the PSF license.
 #
 #       Copyright (C) 2001 Python Software Foundation, www.python.org
 #       Copyright (C) 2005-2006 Fernando Perez. <fperez@colorado.edu>
 #
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
 #
 #*****************************************************************************
 
@@ -49,18 +44,20 @@ from IPython.excolors import ExceptionColors
 # See if we can use pydb.
 has_pydb = False
 prompt = 'ipdb> '
-try:
-    import pydb
-    if hasattr(pydb.pydb, "runl") and pydb.version>'1.17':
-        # Version 1.17 is broken, and that's what ships with Ubuntu Edgy, so we
-        # better protetct against it.
-        has_pydb = True
-        from pydb import Pdb as OldPdb
-except ImportError:
-    pass
+#We have to check this directly from sys.argv, config struct not yet available
+if '-pydb' in sys.argv:
+    try:        
+        import pydb
+        if hasattr(pydb.pydb, "runl") and pydb.version>'1.17':
+            # Version 1.17 is broken, and that's what ships with Ubuntu Edgy, so we
+            # better protect against it.
+            has_pydb = True
+    except ImportError:
+        print "Pydb (http://bashdb.sourceforge.net/pydb/) does not seem to be available"
 
 if has_pydb:
     from pydb import Pdb as OldPdb
+    #print "Using pydb for %run -d and post-mortem" #dbg
     prompt = 'ipydb> '
 else:
     from pdb import Pdb as OldPdb
@@ -72,7 +69,7 @@ def BdbQuit_excepthook(et,ev,tb):
     if et==bdb.BdbQuit:
         print 'Exiting Debugger.'
     else:
-        ehook.excepthook_ori(et,ev,tb)
+        BdbQuit_excepthook.excepthook_ori(et,ev,tb)
 
 def BdbQuit_IPython_excepthook(self,et,ev,tb):
     print 'Exiting Debugger.'
