@@ -44,7 +44,7 @@ each time the instance is evaluated with str(instance).  For example:
     foo = "bar"
     print str(s)
 
-$Id: Itpl.py 958 2005-12-27 23:17:51Z fperez $
+$Id: Itpl.py 2918 2007-12-31 14:34:47Z vivainio $
 """                   # ' -> close an open quote for stupid emacs
 
 #*****************************************************************************
@@ -94,8 +94,16 @@ def matchorfail(text, pos):
     match = tokenprog.match(text, pos)
     if match is None:
         raise ItplError(text, pos)
+
     return match, match.end()
 
+try:
+    itpl_encoding = sys.stdin.encoding or 'ascii'
+except AttributeError:
+    itpl_encoding = 'ascii'
+
+    
+
 class Itpl:
     """Class representing a string with interpolation abilities.
     
@@ -104,7 +112,7 @@ class Itpl:
     evaluation and substitution happens in the namespace of the
     caller when str(instance) is called."""
 
-    def __init__(self, format,codec='utf_8',encoding_errors='backslashreplace'):
+    def __init__(self, format,codec=itpl_encoding,encoding_errors='backslashreplace'):
         """The single mandatory argument to this constructor is a format
         string.
 
@@ -198,7 +206,13 @@ class Itpl:
         result = []
         app = result.append
         for live, chunk in self.chunks:
-            if live: app(str(eval(chunk,glob,loc)))
+            if live:
+                val = eval(chunk,glob,loc)
+                try:
+                    app(str(val))
+                except UnicodeEncodeError:
+                    app(unicode(val))
+                
             else: app(chunk)
         out = ''.join(result)
         try:
