@@ -6,7 +6,7 @@ Uses syntax highlighting for presenting the various information elements.
 Similar in spirit to the inspect module, but all calls take a name argument to
 reference the name under which an object is being read.
 
-$Id: OInspect.py 1850 2006-10-28 19:48:13Z fptest $
+$Id: OInspect.py 2843 2007-10-15 21:22:32Z fperez $
 """
 
 #*****************************************************************************
@@ -150,7 +150,7 @@ def getsource(obj,is_binary=False):
 
     - is_binary: whether the object is known to come from a binary source.
     This implementation will skip returning any output for binary objects, but
-    custom extractors may know how to meaninfully process them."""
+    custom extractors may know how to meaningfully process them."""
     
     if is_binary:
         return None
@@ -234,7 +234,8 @@ class Inspector:
             return
 
         header = ''
-        if type(obj) is types.ClassType:
+
+        if inspect.isclass(obj):
             header = self.__head('Class constructor information:\n')
             obj = obj.__init__
         elif type(obj) is types.InstanceType:
@@ -252,18 +253,19 @@ class Inspector:
         Optional:
         -formatter: a function to run the docstring through for specially
         formatted docstrings."""
-        
+
         head = self.__head  # so that itpl can find it even if private
         ds = getdoc(obj)
         if formatter:
             ds = formatter(ds)
-        if type(obj) is types.ClassType:
+        if inspect.isclass(obj):
             init_ds = getdoc(obj.__init__)
             output = itpl('$head("Class Docstring:")\n'
                           '$indent(ds)\n'
                           '$head("Constructor Docstring"):\n'
                           '$indent(init_ds)')
-        elif type(obj) is types.InstanceType and hasattr(obj,'__call__'):
+        elif (type(obj) is types.InstanceType or isinstance(obj,object)) \
+                 and hasattr(obj,'__call__'):
             call_ds = getdoc(obj.__call__)
             if call_ds:
                 output = itpl('$head("Class Docstring:")\n$indent(ds)\n'
@@ -337,7 +339,15 @@ class Inspector:
             ospace = info.namespace
         # Get docstring, special-casing aliases:
         if isalias:
-            ds = "Alias to the system command:\n  %s" % obj[1]
+            if not callable(obj):
+                try:
+                    ds = "Alias to the system command:\n  %s" % obj[1]
+                except:
+                    ds = "Alias: " + str(obj)
+            else:
+                ds = "Alias to " + str(obj)
+                if obj.__doc__:
+                    ds += "\nDocstring:\n" + obj.__doc__
         else:
             ds = getdoc(obj)
             if ds is None:
@@ -394,11 +404,16 @@ class Inspector:
         # Filename where object was defined
         binary_file = False
         try:
-            fname = inspect.getabsfile(obj)
+            try:
+                fname = inspect.getabsfile(obj)
+            except TypeError:
+                # For an instance, the file that matters is where its class was
+                # declared.
+                if hasattr(obj,'__class__'):
+                    fname = inspect.getabsfile(obj.__class__)
             if fname.endswith('<string>'):
                 fname = 'Dynamically generated function. No source code available.'
-            if (fname.endswith('.so') or fname.endswith('.dll') or 
-                not os.path.isfile(fname)):
+            if (fname.endswith('.so') or fname.endswith('.dll')):
                 binary_file = True
             out.writeln(header('File:\t\t')+fname)
         except:
@@ -415,16 +430,21 @@ class Inspector:
         # avoid repetitions).  If source fails, we add them back, see below.
         if ds and detail_level == 0:
                 out.writeln(header('Docstring:\n') + indent(ds))
-
                 
         # Original source code for any callable
         if detail_level:
-            # Flush the source cache because inspect can return out-of-date source
+            # Flush the source cache because inspect can return out-of-date
+            # source
             linecache.checkcache()
             source_success = False
             try:
-                source = self.format(getsource(obj,binary_file))
-                if source:
+                try:
+                    src = getsource(obj,binary_file)
+                except TypeError:
+                    if hasattr(obj,'__class__'):
+                        src = getsource(obj.__class__,binary_file)
+                if src is not None:
+                    source = self.format(src)
                     out.write(header('Source:\n')+source.rstrip())
                     source_success = True
             except Exception, msg:
@@ -435,7 +455,7 @@ class Inspector:
                             + indent(ds))
 
         # Constructor docstring for classes
-        if obj_type is types.ClassType:
+        if inspect.isclass(obj):
             # reconstruct the function definition and print it:
             try:
                 obj_init =  obj.__init__
@@ -444,6 +464,10 @@ class Inspector:
             else:
                 init_def = self.__getdef(obj_init,oname)
                 init_ds  = getdoc(obj_init)
+                # Skip Python's auto-generated docstrings
+                if init_ds and \
+                       init_ds.startswith('x.__init__(...) initializes'):
+                    init_ds = None
 
             if init_def or init_ds:
                 out.writeln(header('\nConstructor information:'))
@@ -452,14 +476,26 @@ class Inspector:
                 if init_ds:
                     out.writeln(header('Docstring:\n') + indent(init_ds))
         # and class docstring for instances:
-        elif obj_type is types.InstanceType:
+        elif obj_type is types.InstanceType or \
+                 isinstance(obj,object):
 
             # First, check whether the instance docstring is identical to the
             # class one, and print it separately if they don't coincide.  In
             # most cases they will, but it's nice to print all the info for
             # objects which use instance-customized docstrings.
             if ds:
-                class_ds = getdoc(obj.__class__)
+                try:
+                    cls = getattr(obj,'__class__')
+                except:
+                    class_ds = None
+                else:
+                    class_ds = getdoc(cls)
+                # Skip Python's auto-generated docstrings
+                if class_ds and \
+                       (class_ds.startswith('function(code, globals[,') or \
+                   class_ds.startswith('instancemethod(function, instance,') or \
+                   class_ds.startswith('module(name[,') ):
+                    class_ds = None
                 if class_ds and ds != class_ds:
                     out.writeln(header('Class Docstring:\n') +
                                 indent(class_ds))
@@ -467,6 +503,10 @@ class Inspector:
             # Next, try to show constructor docstrings
             try:
                 init_ds = getdoc(obj.__init__)
+                # Skip Python's auto-generated docstrings
+                if init_ds and \
+                       init_ds.startswith('x.__init__(...) initializes'):
+                    init_ds = None
             except AttributeError:
                 init_ds = None
             if init_ds:
@@ -475,14 +515,17 @@ class Inspector:
 
             # Call form docstring for callable instances
             if hasattr(obj,'__call__'):
-                out.writeln(header('Callable:\t')+'Yes')
+                #out.writeln(header('Callable:\t')+'Yes')
                 call_def = self.__getdef(obj.__call__,oname)
-                if call_def is None:
-                    out.write(header('Call def:\t')+
-                              'Calling definition not available.')
-                else:
-                    out.write(header('Call def:\t')+self.format(call_def))
+                #if call_def is None:
+                #    out.writeln(header('Call def:\t')+
+                #              'Calling definition not available.')
+                if call_def is not None:
+                    out.writeln(header('Call def:\t')+self.format(call_def))
                 call_ds = getdoc(obj.__call__)
+                # Skip Python's auto-generated docstrings
+                if call_ds and call_ds.startswith('x.__call__(...) <==> x(...)'):
+                    call_ds = None
                 if call_ds:
                     out.writeln(header('Call docstring:\n') + indent(call_ds))
 
@@ -513,6 +556,8 @@ class Inspector:
           - show_all(False): show all names, including those starting with
           underscores.
         """
+        #print 'ps pattern:<%r>' % pattern # dbg
+        
         # defaults
         type_pattern = 'all'
         filter = ''
