@@ -2,7 +2,7 @@
 """
 Classes for handling input/output prompts.
 
-$Id: Prompts.py 2192 2007-04-01 20:51:06Z fperez $"""
+$Id: Prompts.py 3026 2008-02-07 16:03:16Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
@@ -30,6 +30,7 @@ from IPython.Itpl import ItplNS
 from IPython.ipstruct import Struct
 from IPython.macro import Macro
 from IPython.genutils import *
+from IPython.ipapi import TryNext
 
 #****************************************************************************
 #Color schemes for Prompts.
@@ -214,8 +215,19 @@ def str_safe(arg):
         out = '<ERROR: %s>' % msg
     return out
 
-class BasePrompt:
+class BasePrompt(object):
     """Interactive prompt similar to Mathematica's."""
+
+    def _get_p_template(self):
+        return self._p_template
+
+    def _set_p_template(self,val):
+        self._p_template = val
+        self.set_p_str()
+
+    p_template = property(_get_p_template,_set_p_template,
+                          doc='Template for prompt string creation')
+
     def __init__(self,cache,sep,prompt,pad_left=False):
 
         # Hack: we access information about the primary prompt through the
@@ -231,7 +243,9 @@ class BasePrompt:
         # Flag to left-pad prompt strings to match the length of the primary
         # prompt
         self.pad_left = pad_left
-        # Set template to create each actual prompt (where numbers change)
+
+        # Set template to create each actual prompt (where numbers change).
+        # Use a property
         self.p_template = prompt
         self.set_p_str()
 
@@ -243,14 +257,19 @@ class BasePrompt:
 
         import os,time  # needed in locals for prompt string handling
         loc = locals()
-        self.p_str = ItplNS('%s%s%s' %
-                            ('${self.sep}${self.col_p}',
-                             multiple_replace(prompt_specials, self.p_template),
-                             '${self.col_norm}'),self.cache.user_ns,loc)
-
-        self.p_str_nocolor = ItplNS(multiple_replace(prompt_specials_nocolor,
-                                                     self.p_template),
-                                    self.cache.user_ns,loc)
+        try:
+            self.p_str = ItplNS('%s%s%s' %
+                                ('${self.sep}${self.col_p}',
+                                 multiple_replace(prompt_specials, self.p_template),
+                                 '${self.col_norm}'),self.cache.user_ns,loc)
+    
+            self.p_str_nocolor = ItplNS(multiple_replace(prompt_specials_nocolor,
+                                                         self.p_template),
+                                        self.cache.user_ns,loc)
+        except:
+            print "Illegal prompt template (check $ usage!):",self.p_template
+            self.p_str = self.p_template
+            self.p_str_nocolor = self.p_template
 
     def write(self,msg):  # dbg
         sys.stdout.write(msg)
@@ -295,15 +314,27 @@ class BasePrompt:
         $HOME is always replaced with '~'.
         If depth==0, the full path is returned."""
 
-        cwd = os.getcwd().replace(HOME,"~").split(os.sep)
+        full_cwd = os.getcwd()
+        cwd = full_cwd.replace(HOME,"~").split(os.sep)
         if '~' in cwd and len(cwd) == depth+1:
             depth += 1
-        out = os.sep.join(cwd[-depth:])
+        drivepart = ''
+        if sys.platform == 'win32' and len(cwd) > depth:
+            drivepart = os.path.splitdrive(full_cwd)[0]
+        out = drivepart + '/'.join(cwd[-depth:])
+
         if out:
             return out
         else:
             return os.sep
 
+    def __nonzero__(self):
+        """Implement boolean behavior.
+
+        Checks whether the p_str attribute is non-empty"""
+
+        return bool(self.p_template)
+
 class Prompt1(BasePrompt):
     """Input interactive prompt similar to Mathematica's."""
 
@@ -504,21 +535,18 @@ class CachedOutput:
             # first handle the cache and counters
 
             # do not print output if input ends in ';'
-            if self.input_hist[self.prompt_count].endswith(';\n'):
-                return
+            try:
+                if self.input_hist[self.prompt_count].endswith(';\n'):
+                    return
+            except IndexError:
+                # some uses of ipshellembed may fail here
+                pass
             # don't use print, puts an extra space
             cout_write(self.output_sep)
             outprompt = self.shell.hooks.generate_output_prompt()
             if self.do_full_cache:
                 cout_write(outprompt)
 
-            if isinstance(arg,Macro):
-                print 'Executing Macro...'
-                # in case the macro takes a long time to execute
-                Term.cout.flush()
-                self.shell.runlines(arg.value)
-                return None
-
             # and now call a possibly user-defined print mechanism
             manipulated_val = self.display(arg)
             
@@ -543,8 +571,10 @@ class CachedOutput:
         Do ip.set_hook("result_display", my_displayhook) for custom result
         display, e.g. when your own objects need special formatting.
         """
-
-        return self.shell.hooks.result_display(arg)
+        try:
+            return IPython.generics.result_display(arg)
+        except TryNext:            
+            return self.shell.hooks.result_display(arg)
 
     # Assign the default display method:
     display = _display
@@ -572,8 +602,8 @@ class CachedOutput:
         if self.do_full_cache:
             new_result = '_'+`self.prompt_count`
             to_main[new_result] = arg
-        self.user_ns.update(to_main)
-        self.user_ns['_oh'][self.prompt_count] = arg
+            self.user_ns.update(to_main)
+            self.user_ns['_oh'][self.prompt_count] = arg
 
     def flush(self):
         if not self.do_full_cache:
