@@ -11,7 +11,7 @@
     
     MoinMoin - Python Source Parser
 
-    Title:olorize Python source using the built-in tokenizer
+    Title: Colorize Python source using the built-in tokenizer
            
     Submitter: Jurgen Hermann
     Last Updated:2001/04/06
@@ -28,7 +28,7 @@
     scan Python source code and re-emit it with no changes to its original
     formatting (which is the hard part).
 
-    $Id: PyColorize.py 2274 2007-04-26 14:41:43Z jdh2358 $"""
+    $Id: PyColorize.py 2586 2007-08-06 19:30:09Z vivainio $"""
 
 __all__ = ['ANSICodeColors','Parser']
 
@@ -38,6 +38,7 @@ _scheme_default = 'Linux'
 import cStringIO
 import keyword
 import os
+import optparse
 import string
 import sys
 import token
@@ -242,46 +243,59 @@ class Parser:
         # send text
         owrite('%s%s%s' % (color,toktext,colors.normal))
             
-def main():
-    """Colorize a python file using ANSI color escapes and print to stdout.
+def main(argv=None):
+    """Run as a command-line script: colorize a python file or stdin using ANSI
+    color escapes and print to stdout.
 
-    Usage:
-      %s [-s scheme] filename
+    Inputs:
 
-    Options:
+      - argv(None): a list of strings like sys.argv[1:] giving the command-line
+        arguments. If None, use sys.argv[1:].
+    """
 
-      -s scheme: give the color scheme to use. Currently only 'Linux'
-      (default) and 'LightBG' and 'NoColor' are implemented (give without
-      quotes).  """  
+    usage_msg = """%prog [options] [filename]
 
-    def usage():
-        print >> sys.stderr, main.__doc__ % sys.argv[0]
-        sys.exit(1)
-        
-    # FIXME: rewrite this to at least use getopt
-    try:
-        if sys.argv[1] == '-s':
-            scheme_name = sys.argv[2]
-            del sys.argv[1:3]
-        else:
-            scheme_name = _scheme_default
-        
-    except:
-        usage()
+Colorize a python file or stdin using ANSI color escapes and print to stdout.
+If no filename is given, or if filename is -, read standard input."""
+
+    parser = optparse.OptionParser(usage=usage_msg)
+    newopt = parser.add_option
+    newopt('-s','--scheme',metavar='NAME',dest='scheme_name',action='store',
+           choices=['Linux','LightBG','NoColor'],default=_scheme_default,
+           help="give the color scheme to use. Currently only 'Linux'\
+ (default) and 'LightBG' and 'NoColor' are implemented (give without\
+ quotes)")
+
+    opts,args = parser.parse_args(argv)
+
+    if len(args) > 1:
+        parser.error("you must give at most one filename.")
+
+    if len(args) == 0:
+        fname = '-' # no filename given; setup to read from stdin
+    else:
+        fname = args[0]
+
+    if fname == '-':
+        stream = sys.stdin
+    else:
+        stream = file(fname)
 
-    try:
-        fname = sys.argv[1]
-    except:
-        usage()
-        
-    # write colorized version to stdout
     parser = Parser()
+
+    # we need nested try blocks because pre-2.5 python doesn't support unified
+    # try-except-finally
     try:
-        parser.format(file(fname).read(),scheme = scheme_name)
-    except IOError,msg:
-        # if user reads through a pager and quits, don't print traceback
-        if msg.args != (32,'Broken pipe'):
-            raise
+        try:
+            # write colorized version to stdout
+            parser.format(stream.read(),scheme=opts.scheme_name)
+        except IOError,msg:
+            # if user reads through a pager and quits, don't print traceback
+            if msg.args != (32,'Broken pipe'):
+                raise
+    finally:
+        if stream is not sys.stdin:
+            stream.close() # in case a non-handled exception happened above
 
 if __name__ == "__main__":
     main()
