@@ -4,7 +4,7 @@
 All the matplotlib support code was co-developed with John Hunter,
 matplotlib's author.
 
-$Id: Shell.py 2222 2007-04-06 17:11:27Z fperez $"""
+$Id: Shell.py 3024 2008-02-07 15:34:42Z darren.dale $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
@@ -39,13 +39,20 @@ except ImportError:
 
 # IPython imports
 import IPython
-from IPython import ultraTB
-from IPython.genutils import Term,warn,error,flag_calls
+from IPython import ultraTB, ipapi
+from IPython.genutils import Term,warn,error,flag_calls, ask_yes_no
 from IPython.iplib import InteractiveShell
 from IPython.ipmaker import make_IPython
 from IPython.Magic import Magic
 from IPython.ipstruct import Struct
 
+try: # Python 2.3 compatibility
+    set
+except NameError:
+    import sets
+    set = sets.Set
+
+
 # Globals
 # global flag to pass around information about Ctrl-C without exceptions
 KBINT = False
@@ -79,6 +86,22 @@ class IPShell:
             sys.exit()
 
 #-----------------------------------------------------------------------------
+def kill_embedded(self,parameter_s=''):
+    """%kill_embedded : deactivate for good the current embedded IPython.
+
+    This function (after asking for confirmation) sets an internal flag so that
+    an embedded IPython will never activate again.  This is useful to
+    permanently disable a shell that is being called inside a loop: once you've
+    figured out what you needed from it, you may then kill it and the program
+    will then continue to run without the interactive shell interfering again.
+    """
+    
+    kill = ask_yes_no("Are you sure you want to kill this embedded instance "
+                     "(y/n)? [y/N] ",'n')
+    if kill:
+        self.shell.embedded_active = False
+        print "This embedded IPython will not reactivate anymore once you exit."
+    
 class IPShellEmbed:
     """Allow embedding an IPython shell into a running program.
 
@@ -153,6 +176,9 @@ class IPShellEmbed:
                                embedded=True,
                                user_ns=user_ns)
 
+        ip = ipapi.IPApi(self.IP)
+        ip.expose_magic("kill_embedded",kill_embedded)
+
         # copy our own displayhook also
         self.sys_displayhook_embed = sys.displayhook
         # and leave the system's display hook clean
@@ -196,6 +222,14 @@ class IPShellEmbed:
         The optional keyword parameter dummy controls whether the call
         actually does anything.  """
 
+        # If the user has turned it off, go away
+        if not self.IP.embedded_active:
+            return
+
+        # Normal exits from interactive mode set this flag, so the shell can't
+        # re-enter (it checks this variable at the start of interactive mode).
+        self.IP.exit_now = False
+
         # Allow the dummy parameter to override the global __dummy_mode
         if dummy or (dummy != 0 and self.__dummy_mode):
             return
@@ -331,38 +365,37 @@ class MTInteractiveShell(InteractiveShell):
         InteractiveShell.__init__(self,name,usage,rc,user_ns,
                                   user_global_ns,banner2)
 
-        # Locking control variable.  We need to use a norma lock, not an RLock
-        # here.  I'm not exactly sure why, it seems to me like it should be
-        # the opposite, but we deadlock with an RLock.  Puzzled...
-        self.thread_ready = threading.Condition(threading.Lock())
 
-        # A queue to hold the code to be executed.  A scalar variable is NOT
-        # enough, because uses like macros cause reentrancy.
+        # A queue to hold the code to be executed. 
         self.code_queue = Queue.Queue()
 
         # Stuff to do at closing time
-        self._kill = False
-        on_kill = kw.get('on_kill')
-        if on_kill is None:
-            on_kill = []
+        self._kill = None
+        on_kill = kw.get('on_kill', [])
         # Check that all things to kill are callable:
         for t in on_kill:
             if not callable(t):
                 raise TypeError,'on_kill must be a list of callables'
         self.on_kill = on_kill
-
+        # thread identity of the "worker thread" (that may execute code directly)
+        self.worker_ident = None
+        
     def runsource(self, source, filename="<input>", symbol="single"):
         """Compile and run some source in the interpreter.
 
         Modified version of code.py's runsource(), to handle threading issues.
         See the original for full docstring details."""
-
+        
         global KBINT
         
         # If Ctrl-C was typed, we reset the flag and return right away
         if KBINT:
             KBINT = False
             return False
+
+        if self._kill:
+            # can't queue new code if we are being killed
+            return True
         
         try:
             code = self.compile(source, filename, symbol)
@@ -375,33 +408,40 @@ class MTInteractiveShell(InteractiveShell):
             # Case 2
             return True
 
+        # shortcut - if we are in worker thread, or the worker thread is not running, 
+        # execute directly (to allow recursion and prevent deadlock if code is run early 
+        # in IPython construction)
+        
+        if (self.worker_ident is None or self.worker_ident == thread.get_ident()):
+            InteractiveShell.runcode(self,code)
+            return
+
         # Case 3
         # Store code in queue, so the execution thread can handle it.
 
-        # Note that with macros and other applications, we MAY re-enter this
-        # section, so we have to acquire the lock with non-blocking semantics,
-        # else we deadlock.
-        got_lock = self.thread_ready.acquire(False)
-        self.code_queue.put(code)
-        if got_lock:
-            self.thread_ready.wait()  # Wait until processed in timeout interval
-            self.thread_ready.release()
-
+        completed_ev, received_ev = threading.Event(), threading.Event() 
+        
+        self.code_queue.put((code,completed_ev, received_ev))
+        # first make sure the message was received, with timeout
+        received_ev.wait(5)
+        if not received_ev.isSet():
+            # the mainloop is dead, start executing code directly
+            print "Warning: Timeout for mainloop thread exceeded"
+            print "switching to nonthreaded mode (until mainloop wakes up again)"
+            self.worker_ident = None
+        else:
+            completed_ev.wait()
         return False
 
     def runcode(self):
         """Execute a code object.
 
         Multithreaded wrapper around IPython's runcode()."""
-
-        global CODE_RUN
-
-        # Exceptions need to be raised differently depending on which thread is
-        # active
-        CODE_RUN = True
         
-        # lock thread-protected stuff
-        got_lock = self.thread_ready.acquire(False)
+        global CODE_RUN
+        
+        # we are in worker thread, stash out the id for runsource() 
+        self.worker_ident = thread.get_ident()
 
         if self._kill:
             print >>Term.cout, 'Closing threads...',
@@ -409,6 +449,9 @@ class MTInteractiveShell(InteractiveShell):
             for tokill in self.on_kill:
                 tokill()
             print >>Term.cout, 'Done.'
+            # allow kill() to return
+            self._kill.set()
+            return True
 
         # Install sigint handler.  We do it every time to ensure that if user
         # code modifies it, we restore our own handling.
@@ -421,32 +464,44 @@ class MTInteractiveShell(InteractiveShell):
 
         # Flush queue of pending code by calling the run methood of the parent
         # class with all items which may be in the queue.
+        code_to_run = None
         while 1:
             try:
-                code_to_run = self.code_queue.get_nowait()
+                code_to_run, completed_ev, received_ev = self.code_queue.get_nowait()                
             except Queue.Empty:
                 break
-            if got_lock:
-                self.thread_ready.notify()
-                InteractiveShell.runcode(self,code_to_run)
-            else:
-                break
+            received_ev.set()
             
-        # We're done with thread-protected variables
-        if got_lock:
-            self.thread_ready.release()
-
-        # We're done...
-        CODE_RUN = False
+            # Exceptions need to be raised differently depending on which
+            # thread is active.  This convoluted try/except is only there to
+            # protect against asynchronous exceptions, to ensure that a KBINT
+            # at the wrong time doesn't deadlock everything.  The global
+            # CODE_TO_RUN is set to true/false as close as possible to the
+            # runcode() call, so that the KBINT handler is correctly informed.
+            try:
+                try:
+                   CODE_RUN = True
+                   InteractiveShell.runcode(self,code_to_run)
+                except KeyboardInterrupt:
+                   print "Keyboard interrupted in mainloop"
+                   while not self.code_queue.empty():
+                      code, ev1,ev2 = self.code_queue.get_nowait()
+                      ev1.set()
+                      ev2.set()                      
+                   break
+            finally:
+                CODE_RUN = False
+                # allow runsource() return from wait
+                completed_ev.set()
+                
+        
         # This MUST return true for gtk threading to work
         return True
 
     def kill(self):
         """Kill the thread, returning when it has been shut down."""
-        got_lock = self.thread_ready.acquire(False)
-        self._kill = True
-        if got_lock:
-            self.thread_ready.release()
+        self._kill = threading.Event()
+        self._kill.wait()
 
 class MatplotlibShellBase:
     """Mixin class to provide the necessary modifications to regular IPython
@@ -657,6 +712,33 @@ def hijack_gtk():
     gtk.main = dummy_mainloop
     return orig_mainloop
 
+def hijack_qt():
+    """Modifies PyQt's mainloop with a dummy so user code does not 
+    block IPython.  This function returns the original 
+    `qt.qApp.exec_loop` function that has been hijacked.
+    """    
+    def dummy_mainloop(*args, **kw):
+        pass
+    import qt
+    orig_mainloop = qt.qApp.exec_loop
+    qt.qApp.exec_loop = dummy_mainloop
+    qt.QApplication.exec_loop = dummy_mainloop
+    return orig_mainloop
+
+def hijack_qt4():
+    """Modifies PyQt4's mainloop with a dummy so user code does not
+    block IPython.  This function returns the original 
+    `QtGui.qApp.exec_` function that has been hijacked.
+    """    
+    def dummy_mainloop(*args, **kw):
+        pass
+    from PyQt4 import QtGui, QtCore
+    orig_mainloop = QtGui.qApp.exec_
+    QtGui.qApp.exec_ = dummy_mainloop
+    QtGui.QApplication.exec_ = dummy_mainloop
+    QtCore.QCoreApplication.exec_ = dummy_mainloop
+    return orig_mainloop
+
 #-----------------------------------------------------------------------------
 # The IPShell* classes below are the ones meant to be run by external code as
 # IPython instances.  Note that unless a specific threading strategy is
@@ -852,30 +934,18 @@ class IPShellQt(IPThread):
     
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
-                 debug=0,shell_class=MTInteractiveShell):
-        
+    def __init__(self, argv=None, user_ns=None, user_global_ns=None,
+                 debug=0, shell_class=MTInteractiveShell):
+
         import qt
 
-        class newQApplication:
-            def __init__( self ):
-                self.QApplication = qt.QApplication
-                
-            def __call__( *args, **kwargs ):
-                return qt.qApp
-
-            def exec_loop( *args, **kwargs ):
-                pass
-
-            def __getattr__( self, name ):
-                return getattr( self.QApplication, name )
-          
-        qt.QApplication = newQApplication()
+        self.exec_loop = hijack_qt()
 
         # Allows us to use both Tk and QT.
         self.tk = get_tk()
 
-        self.IP = make_IPython(argv,user_ns=user_ns,
+        self.IP = make_IPython(argv,
+                               user_ns=user_ns,
                                user_global_ns=user_global_ns,
                                debug=debug,
                                shell_class=shell_class,
@@ -888,28 +958,31 @@ class IPShellQt(IPThread):
         
         threading.Thread.__init__(self)
 
-    def mainloop(self,sys_exit=0,banner=None):
+    def mainloop(self, sys_exit=0, banner=None):
 
         import qt
 
         self._banner = banner
 
         if qt.QApplication.startingUp():
-          a = qt.QApplication.QApplication(sys.argv)
+            a = qt.QApplication(sys.argv)
+
         self.timer = qt.QTimer()
-        qt.QObject.connect( self.timer, qt.SIGNAL( 'timeout()' ), self.on_timer )
+        qt.QObject.connect(self.timer,
+                           qt.SIGNAL('timeout()'),
+                           self.on_timer)
 
         self.start()
-        self.timer.start( self.TIMEOUT, True )
+        self.timer.start(self.TIMEOUT, True)
         while True:
             if self.IP._kill: break
-            qt.qApp.exec_loop()
+            self.exec_loop()
         self.join()
 
     def on_timer(self):
         update_tk(self.tk)
         result = self.IP.runcode()
-        self.timer.start( self.TIMEOUT, True )
+        self.timer.start(self.TIMEOUT, True)
         return result
 
 
@@ -922,30 +995,29 @@ class IPShellQt4(IPThread):
 
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
-                 debug=0,shell_class=MTInteractiveShell):
+    def __init__(self, argv=None, user_ns=None, user_global_ns=None,
+                 debug=0, shell_class=MTInteractiveShell):
 
         from PyQt4 import QtCore, QtGui
 
-        class newQApplication:
-            def __init__( self ):
-                self.QApplication = QtGui.QApplication
+        try:
+            # present in PyQt4-4.2.1 or later
+            QtCore.pyqtRemoveInputHook()
+        except AttributeError:
+            pass
 
-            def __call__( *args, **kwargs ):
-                return QtGui.qApp
+        if QtCore.PYQT_VERSION_STR == '4.3':
+            warn('''PyQt4 version 4.3 detected.
+If you experience repeated threading warnings, please update PyQt4.
+''')
 
-            def exec_loop( *args, **kwargs ):
-                pass
-
-            def __getattr__( self, name ):
-                return getattr( self.QApplication, name )
-
-        QtGui.QApplication = newQApplication()
+        self.exec_ = hijack_qt4()
 
         # Allows us to use both Tk and QT.
         self.tk = get_tk()
 
-        self.IP = make_IPython(argv,user_ns=user_ns,
+        self.IP = make_IPython(argv,
+                               user_ns=user_ns,
                                user_global_ns=user_global_ns,
                                debug=debug,
                                shell_class=shell_class,
@@ -958,28 +1030,31 @@ class IPShellQt4(IPThread):
 
         threading.Thread.__init__(self)
 
-    def mainloop(self,sys_exit=0,banner=None):
+    def mainloop(self, sys_exit=0, banner=None):
 
         from PyQt4 import QtCore, QtGui
 
         self._banner = banner
 
         if QtGui.QApplication.startingUp():
-          a = QtGui.QApplication.QApplication(sys.argv)
+            a = QtGui.QApplication(sys.argv)
+
         self.timer = QtCore.QTimer()
-        QtCore.QObject.connect( self.timer, QtCore.SIGNAL( 'timeout()' ), self.on_timer )
+        QtCore.QObject.connect(self.timer,
+                               QtCore.SIGNAL('timeout()'),
+                               self.on_timer)
 
         self.start()
-        self.timer.start( self.TIMEOUT )
+        self.timer.start(self.TIMEOUT)
         while True:
             if self.IP._kill: break
-            QtGui.qApp.exec_()
+            self.exec_()
         self.join()
 
     def on_timer(self):
         update_tk(self.tk)
         result = self.IP.runcode()
-        self.timer.start( self.TIMEOUT )
+        self.timer.start(self.TIMEOUT)
         return result
 
 
@@ -994,7 +1069,9 @@ def _load_pylab(user_ns):
     
     ip = IPython.ipapi.get()
     if ip.options.pylab_import_all:
-        exec "from matplotlib.pylab import *" in user_ns
+        ip.ex("from matplotlib.pylab import *")
+        ip.IP.user_config_ns.update(ip.user_ns)
+        
 
 class IPShellMatplotlib(IPShell):
     """Subclass IPShell with MatplotlibShell as the internal shell.
@@ -1051,31 +1128,85 @@ class IPShellMatplotlibQt4(IPShellQt4):
 #-----------------------------------------------------------------------------
 # Factory functions to actually start the proper thread-aware shell
 
-def _matplotlib_shell_class():
-    """Factory function to handle shell class selection for matplotlib.
+def _select_shell(argv):
+    """Select a shell from the given argv vector.
 
-    The proper shell class to use depends on the matplotlib backend, since
-    each backend requires a different threading strategy."""
+    This function implements the threading selection policy, allowing runtime
+    control of the threading mode, both for general users and for matplotlib.
 
-    try:
-        import matplotlib
-    except ImportError:
-        error('matplotlib could NOT be imported!  Starting normal IPython.')
-        sh_class = IPShell
-    else:
-        backend = matplotlib.rcParams['backend']
-        if backend.startswith('GTK'):
-            sh_class = IPShellMatplotlibGTK
-        elif backend.startswith('WX'):
-            sh_class = IPShellMatplotlibWX
-        elif backend.startswith('Qt4'):
-            sh_class = IPShellMatplotlibQt4
-        elif backend.startswith('Qt'):
-            sh_class = IPShellMatplotlibQt
+    Return:
+      Shell class to be instantiated for runtime operation.
+    """
+    
+    global USE_TK
+
+    mpl_shell = {'gthread' : IPShellMatplotlibGTK,
+                 'wthread' : IPShellMatplotlibWX,
+                 'qthread' : IPShellMatplotlibQt,
+                 'q4thread' : IPShellMatplotlibQt4,
+                 'tkthread' : IPShellMatplotlib,  # Tk is built-in
+                 }
+
+    th_shell = {'gthread' : IPShellGTK,
+                'wthread' : IPShellWX,
+                'qthread' : IPShellQt,
+                'q4thread' : IPShellQt4,
+                'tkthread' : IPShell, # Tk is built-in
+                }
+
+    backends = {'gthread' : 'GTKAgg',
+                'wthread' : 'WXAgg',
+                'qthread' : 'QtAgg',
+                'q4thread' :'Qt4Agg',
+                'tkthread' :'TkAgg',
+                }
+
+    all_opts = set(['tk','pylab','gthread','qthread','q4thread','wthread',
+                    'tkthread'])
+    user_opts = set([s.replace('-','') for s in argv[:3]])
+    special_opts = user_opts & all_opts            
+
+    if 'tk' in special_opts:
+        USE_TK = True
+        special_opts.remove('tk')
+
+    if 'pylab' in special_opts:
+
+        try:
+            import matplotlib
+        except ImportError:
+            error('matplotlib could NOT be imported!  Starting normal IPython.')
+            return IPShell
+        
+        special_opts.remove('pylab')
+        # If there's any option left, it means the user wants to force the
+        # threading backend, else it's auto-selected from the rc file
+        if special_opts:
+            th_mode = special_opts.pop()
+            matplotlib.rcParams['backend'] = backends[th_mode]
         else:
-            sh_class = IPShellMatplotlib
-    #print 'Using %s with the %s backend.' % (sh_class,backend) # dbg
-    return sh_class
+            backend = matplotlib.rcParams['backend']
+            if backend.startswith('GTK'):
+                th_mode = 'gthread'
+            elif backend.startswith('WX'):
+                th_mode = 'wthread'
+            elif backend.startswith('Qt4'):
+                th_mode = 'q4thread'
+            elif backend.startswith('Qt'):
+                th_mode = 'qthread'
+            else:
+                # Any other backend, use plain Tk
+                th_mode = 'tkthread'
+                
+        return mpl_shell[th_mode]
+    else:
+        # No pylab requested, just plain threads
+        try:
+            th_mode = special_opts.pop()
+        except KeyError:
+            th_mode = 'tkthread'
+        return th_shell[th_mode]
+
 
 # This is the one which should be called by external code.
 def start(user_ns = None):
@@ -1085,29 +1216,7 @@ def start(user_ns = None):
     based on the user's threading choice.  Such a selector is needed because
     different GUI toolkits require different thread handling details."""
 
-    global USE_TK
-    # Crude sys.argv hack to extract the threading options.
-    argv = sys.argv
-    if len(argv) > 1:
-        if len(argv) > 2:
-            arg2 = argv[2]
-            if arg2.endswith('-tk'):
-                USE_TK = True
-        arg1 = argv[1]
-        if arg1.endswith('-gthread'):
-            shell = IPShellGTK
-        elif arg1.endswith( '-qthread' ):
-            shell = IPShellQt
-        elif arg1.endswith( '-q4thread' ):
-            shell = IPShellQt4
-        elif arg1.endswith('-wthread'):
-            shell = IPShellWX
-        elif arg1.endswith('-pylab'):
-            shell = _matplotlib_shell_class()
-        else:
-            shell = IPShell
-    else:
-        shell = IPShell
+    shell = _select_shell(sys.argv)
     return shell(user_ns = user_ns)
 
 # Some aliases for backwards compatibility
