@@ -27,7 +27,7 @@ IPython tries to:
 
 IPython requires Python 2.3 or newer.
 
-$Id: __init__.py 1854 2006-10-30 19:54:25Z vivainio $"""
+$Id: __init__.py 2399 2007-05-26 10:23:10Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -48,10 +48,7 @@ import os
 sys.path.append(os.path.dirname(__file__) + "/Extensions")
 
 # Define what gets imported with a 'from IPython import *'
-__all__ = ['deep_reload','genutils','ipstruct','ultraTB','DPyGetOpt',
-           'Itpl','hooks','ConfigLoader','OutputTrap','Release','Shell',
-           'platutils','platutils_win32','platutils_posix','platutils_dummy',
-           'ipapi','rlineimpl', 'strdispatch']
+__all__ = ['ipapi','generics','ipstruct','Release','Shell']
 
 # Load __all__ in IPython namespace so that a simple 'import IPython' gives
 # access to them via IPython.<name>
@@ -59,6 +56,8 @@ glob,loc = globals(),locals()
 for name in __all__:
     __import__(name,glob,loc,[])
 
+import Shell
+
 # Release data
 from IPython import Release # do it explicitly so pydoc can see it - pydoc bug
 __author__   = '%s <%s>\n%s <%s>\n%s <%s>' % \
