@@ -18,11 +18,11 @@ http://folk.uio.no/hpl/scripting
 (although ultimately no code from this text was used, as IPython's system is a
 separate implementation).
 
-$Id: background_jobs.py,v 1.3 2005/02/15 07:41:41 fperez Exp $
+$Id: background_jobs.py 994 2006-01-08 08:29:44Z fperez $
 """
 
 #*****************************************************************************
-#       Copyright (C) 2005 Fernando Perez <fperez@colorado.edu>
+#       Copyright (C) 2005-2006 Fernando Perez <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -33,20 +33,12 @@ __author__  = '%s <%s>' % Release.authors['Fernando']
 __license__ = Release.license
 
 # Code begins
-import threading,sys
+import sys
+import threading
 
 from IPython.ultraTB import AutoFormattedTB
 from IPython.genutils import warn,error
 
-# declares Python 2.2 compatibility symbols:
-try:
-    basestring
-except NameError:
-    import types
-    basestring = (types.StringType, types.UnicodeType)
-    True = 1==1
-    False = 1==0
-
 class BackgroundJobManager:
     """Class to manage a pool of backgrounded threaded jobs.
 
