@@ -5,7 +5,7 @@ General purpose utilities.
 This is a grab-bag of stuff I find useful in most programs I write. Some of
 these things are also convenient when working at the command line.
 
-$Id: genutils.py 2190 2007-03-30 18:35:46Z fperez $"""
+$Id: genutils.py 2998 2008-01-31 10:06:04Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -22,6 +22,10 @@ __license__ = Release.license
 # required modules from the Python standard library
 import __main__
 import commands
+try:
+    import doctest
+except ImportError:
+    pass
 import os
 import re
 import shlex
@@ -32,13 +36,34 @@ import time
 import types
 import warnings
 
+# Curses and termios are Unix-only modules
+try:
+    import curses
+    # We need termios as well, so if its import happens to raise, we bail on
+    # using curses altogether.
+    import termios
+except ImportError:
+    USE_CURSES = False
+else:
+    # Curses on Solaris may not be complete, so we can't use it there
+    USE_CURSES = hasattr(curses,'initscr')
+
 # Other IPython utilities
+import IPython
 from IPython.Itpl import Itpl,itpl,printpl
-from IPython import DPyGetOpt
-from path import path
+from IPython import DPyGetOpt, platutils
+from IPython.generics import result_display
+import IPython.ipapi
+from IPython.external.path import path
 if os.name == "nt":
     from IPython.winconsole import get_console_size
 
+try:
+    set
+except:
+    from sets import Set as set
+
+
 #****************************************************************************
 # Exceptions
 class Error(Exception):
@@ -298,6 +323,25 @@ def system(cmd,verbose=0,debug=0,header=''):
     if not debug: stat = os.system(cmd)
     return stat
 
+def abbrev_cwd():
+    """ Return abbreviated version of cwd, e.g. d:mydir """
+    cwd = os.getcwd().replace('\\','/')
+    drivepart = ''
+    tail = cwd
+    if sys.platform == 'win32':
+        if len(cwd) < 4:
+            return cwd
+        drivepart,tail = os.path.splitdrive(cwd)
+    
+
+    parts = tail.split('/')    
+    if len(parts) > 2:
+        tail = '/'.join(parts[-2:])
+        
+    return (drivepart + (
+        cwd == '/' and '/' or tail))
+    
+
 # This function is used by ipython in a lot of places to make system calls.
 # We need it to be slightly different under win32, due to the vagaries of
 # 'network shares'.  A win32 override is below.
@@ -322,8 +366,11 @@ def shell(cmd,verbose=0,debug=0,header=''):
     if verbose or debug: print header+cmd
     # flush stdout so we don't mangle python's buffering
     sys.stdout.flush()
+    
     if not debug:
+        platutils.set_term_title("IPy " + cmd)
         os.system(cmd)
+        platutils.set_term_title("IPy " + abbrev_cwd())
 
 # override shell() for win32 to deal with network shares
 if os.name in ('nt','dos'):
@@ -601,9 +648,9 @@ def process_cmdline(argv,names=[],defaults={},usage=''):
 
     try:
         getopt.processArguments(argv)
-    except:
+    except DPyGetOpt.ArgumentError, exc:
         print usage
-        warn(`sys.exc_value`,level=4)
+        warn('"%s"' % exc,level=4)
 
     defaults.update(getopt.optionValues)
     args = getopt.freeValues
@@ -793,6 +840,60 @@ def flag_calls(func):
     wrapper.__doc__ = func.__doc__
     return wrapper
 
+#----------------------------------------------------------------------------
+def dhook_wrap(func,*a,**k):
+    """Wrap a function call in a sys.displayhook controller.
+
+    Returns a wrapper around func which calls func, with all its arguments and
+    keywords unmodified, using the default sys.displayhook.  Since IPython
+    modifies sys.displayhook, it breaks the behavior of certain systems that
+    rely on the default behavior, notably doctest.
+    """
+
+    def f(*a,**k):
+    
+        dhook_s = sys.displayhook
+        sys.displayhook = sys.__displayhook__
+        try:
+            out = func(*a,**k)
+        finally:
+            sys.displayhook = dhook_s
+
+        return out
+
+    f.__doc__ = func.__doc__
+    return f
+
+#----------------------------------------------------------------------------
+def doctest_reload():
+    """Properly reload doctest to reuse it interactively.
+
+    This routine:
+
+      - reloads doctest
+      
+      - resets its global 'master' attribute to None, so that multiple uses of
+      the module interactively don't produce cumulative reports.
+    
+      - Monkeypatches its core test runner method to protect it from IPython's
+      modified displayhook.  Doctest expects the default displayhook behavior
+      deep down, so our modification breaks it completely.  For this reason, a
+      hard monkeypatch seems like a reasonable solution rather than asking
+      users to manually use a different doctest runner when under IPython."""
+
+    import doctest
+    reload(doctest)
+    doctest.master=None
+
+    try:
+        doctest.DocTestRunner
+    except AttributeError:
+        # This is only for python 2.3 compatibility, remove once we move to
+        # 2.4 only.
+        pass
+    else:
+        doctest.DocTestRunner.run = dhook_wrap(doctest.DocTestRunner.run)
+
 #----------------------------------------------------------------------------
 class HomeDirError(Error):
     pass
@@ -807,6 +908,16 @@ def get_home_dir():
 
     isdir = os.path.isdir
     env = os.environ
+    
+    # first, check py2exe distribution root directory for _ipython.
+    # This overrides all. Normally does not exist.
+    
+    if '\\library.zip\\' in IPython.__file__.lower():
+        root, rest = IPython.__file__.lower().split('library.zip')
+        if isdir(root + '_ipython'):
+            os.environ["IPYKITROOT"] = root.rstrip('\\')
+            return root
+    
     try:
         homedir = env['HOME']
         if not isdir(homedir):
@@ -863,6 +974,7 @@ class LSString(str):
         .l (or .list) : value as list (split on newlines).
         .n (or .nlstr): original value (the string itself).
         .s (or .spstr): value as whitespace-separated string.
+        .p (or .paths): list of path objects
         
     Any values which require transformations are computed only once and
     cached.
@@ -902,6 +1014,12 @@ class LSString(str):
     
     p = paths = property(get_paths)
 
+def print_lsstring(arg):
+    """ Prettier (non-repr-like) and more informative printer for LSString """
+    print "LSString (.p, .n, .l, .s available). Value:"
+    print arg
+    
+print_lsstring = result_display.when_type(LSString)(print_lsstring)
 
 #----------------------------------------------------------------------------
 class SList(list):
@@ -912,7 +1030,8 @@ class SList(list):
         .l (or .list) : value as list (the list itself).
         .n (or .nlstr): value as a string, joined on newlines.
         .s (or .spstr): value as a string, joined on spaces.
-
+        .p (or .paths): list of path objects
+        
     Any values which require transformations are computed only once and
     cached."""
 
@@ -948,6 +1067,88 @@ class SList(list):
     
     p = paths = property(get_paths)
 
+    def grep(self, pattern, prune = False, field = None):
+        """ Return all strings matching 'pattern' (a regex or callable) 
+        
+        This is case-insensitive. If prune is true, return all items
+        NOT matching the pattern.
+        
+        If field is specified, the match must occur in the specified 
+        whitespace-separated field.
+               
+        Examples::
+        
+            a.grep( lambda x: x.startswith('C') )
+            a.grep('Cha.*log', prune=1)
+            a.grep('chm', field=-1)
+        """
+        
+        def match_target(s):
+            if field is None:
+                return s
+            parts = s.split()
+            try:
+                tgt = parts[field]
+                return tgt
+            except IndexError:
+                return ""
+        
+        if isinstance(pattern, basestring):
+            pred = lambda x : re.search(pattern, x, re.IGNORECASE)
+        else:
+            pred = pattern
+        if not prune:
+            return SList([el for el in self if pred(match_target(el))])
+        else:
+            return SList([el for el in self if not pred(match_target(el))])
+    def fields(self, *fields):
+        """ Collect whitespace-separated fields from string list
+        
+        Allows quick awk-like usage of string lists.
+        
+        Example data (in var a, created by 'a = !ls -l')::
+            -rwxrwxrwx  1 ville None      18 Dec 14  2006 ChangeLog
+            drwxrwxrwx+ 6 ville None       0 Oct 24 18:05 IPython        
+        
+        a.fields(0) is ['-rwxrwxrwx', 'drwxrwxrwx+']
+        a.fields(1,0) is ['1 -rwxrwxrwx', '6 drwxrwxrwx+']
+        (note the joining by space).
+        a.fields(-1) is ['ChangeLog', 'IPython']
+        
+        IndexErrors are ignored.
+        
+        Without args, fields() just split()'s the strings.
+        """
+        if len(fields) == 0:
+            return [el.split() for el in self]
+        
+        res = SList()
+        for el in [f.split() for f in self]:
+            lineparts = []
+
+            for fd in fields:
+                try:
+                    lineparts.append(el[fd])
+                except IndexError:
+                    pass
+            if lineparts:
+                res.append(" ".join(lineparts))
+            
+        return res        
+                
+            
+            
+        
+
+def print_slist(arg):
+    """ Prettier (non-repr-like) and more informative printer for SList """
+    print "SList (.p, .n, .l, .s, .grep(), .fields() available). Value:"
+    nlprint(arg)
+    
+print_slist = result_display.when_type(SList)(print_slist)
+
+
+
 #----------------------------------------------------------------------------
 def esc_quotes(strng):
     """Return the input string with single and double quotes escaped out"""
@@ -984,7 +1185,7 @@ def make_quoted_expr(s):
     else:
         # give up, backslash-escaped string will do
         return '"%s"' % esc_quotes(s)
-    res = itpl("$raw$quote$s$tailpadding$quote$tail")
+    res = raw + quote + s + tailpadding + quote + tail
     return res
 
 
@@ -1032,7 +1233,7 @@ def raw_input_ext(prompt='',  ps2='... '):
 
 #----------------------------------------------------------------------------
 def ask_yes_no(prompt,default=None):
-    """Asks a question and returns an integer 1/0 (y/n) answer.
+    """Asks a question and returns a boolean (y/n) answer.
 
     If default is given (one of 'y','n'), it is used if the user input is
     empty. Otherwise the question is repeated until an answer is given.
@@ -1328,6 +1529,16 @@ def page(strng,start=0,screen_lines=0,pager_cmd = None):
     written in python, very simplistic.
     """
     
+
+    # first, try the hook
+    ip = IPython.ipapi.get()
+    if ip:
+        try:
+            ip.IP.hooks.show_in_pager(strng)
+            return
+        except IPython.ipapi.TryNext:
+            pass
+        
     # Ugly kludge, but calling curses.initscr() flat out crashes in emacs
     TERM = os.environ.get('TERM','dumb')
     if TERM in ['dumb','emacs'] and os.name != 'nt':
@@ -1352,26 +1563,30 @@ def page(strng,start=0,screen_lines=0,pager_cmd = None):
     # auto-determine screen size
     if screen_lines <= 0:
         if TERM=='xterm':
-            try:
-                import curses
-                if hasattr(curses,'initscr'):
-                    use_curses = 1
-                else:
-                    use_curses = 0
-            except ImportError:
-                use_curses = 0
+            use_curses = USE_CURSES
         else:
             # curses causes problems on many terminals other than xterm.
-            use_curses = 0
+            use_curses = False
         if use_curses:
-                scr = curses.initscr()
-                screen_lines_real,screen_cols = scr.getmaxyx()
-                curses.endwin()
-                screen_lines += screen_lines_real
-                #print '***Screen size:',screen_lines_real,'lines x',\
-                #screen_cols,'columns.' # dbg
+            # There is a bug in curses, where *sometimes* it fails to properly
+            # initialize, and then after the endwin() call is made, the
+            # terminal is left in an unusable state.  Rather than trying to
+            # check everytime for this (by requesting and comparing termios
+            # flags each time), we just save the initial terminal state and
+            # unconditionally reset it every time.  It's cheaper than making
+            # the checks.
+            term_flags = termios.tcgetattr(sys.stdout)
+            scr = curses.initscr()
+            screen_lines_real,screen_cols = scr.getmaxyx()
+            curses.endwin()
+            # Restore terminal state in case endwin() didn't.
+            termios.tcsetattr(sys.stdout,termios.TCSANOW,term_flags)
+            # Now we have what we needed: the screen size in rows/columns
+            screen_lines += screen_lines_real
+            #print '***Screen size:',screen_lines_real,'lines x',\
+            #screen_cols,'columns.' # dbg
         else:
-                screen_lines += screen_lines_def
+            screen_lines += screen_lines_def
 
     #print 'numlines',numlines,'screenlines',screen_lines  # dbg
     if numlines <= screen_lines :
@@ -1439,6 +1654,7 @@ def page_file(fname,start = 0, pager_cmd = None):
         except:
             print 'Unable to show file',`fname`
 
+
 #----------------------------------------------------------------------------
 def snip_print(str,width = 75,print_full = 0,header = ''):
     """Print a string snipping the midsection to fit in width.
@@ -1718,6 +1934,70 @@ def map_method(method,object_list,*argseq,**kw):
         idx += 1
     return out_list
 
+#----------------------------------------------------------------------------
+def get_class_members(cls):
+    ret = dir(cls)
+    if hasattr(cls,'__bases__'):
+        for base in cls.__bases__:
+            ret.extend(get_class_members(base))
+    return ret
+
+#----------------------------------------------------------------------------
+def dir2(obj):
+    """dir2(obj) -> list of strings
+
+    Extended version of the Python builtin dir(), which does a few extra
+    checks, and supports common objects with unusual internals that confuse
+    dir(), such as Traits and PyCrust.
+
+    This version is guaranteed to return only a list of true strings, whereas
+    dir() returns anything that objects inject into themselves, even if they
+    are later not really valid for attribute access (many extension libraries
+    have such bugs).
+    """
+    
+    # Start building the attribute list via dir(), and then complete it
+    # with a few extra special-purpose calls.
+    words = dir(obj)
+
+    if hasattr(obj,'__class__'):
+        words.append('__class__')
+        words.extend(get_class_members(obj.__class__))
+    #if '__base__' in words: 1/0
+
+    # Some libraries (such as traits) may introduce duplicates, we want to
+    # track and clean this up if it happens
+    may_have_dupes = False
+
+    # this is the 'dir' function for objects with Enthought's traits
+    if hasattr(obj, 'trait_names'):
+        try:
+            words.extend(obj.trait_names())
+            may_have_dupes = True
+        except TypeError:
+            # This will happen if `obj` is a class and not an instance.
+            pass
+
+    # Support for PyCrust-style _getAttributeNames magic method.
+    if hasattr(obj, '_getAttributeNames'):
+        try:
+            words.extend(obj._getAttributeNames())
+            may_have_dupes = True
+        except TypeError:
+            # `obj` is a class and not an instance.  Ignore
+            # this error.
+            pass
+
+    if may_have_dupes:
+        # eliminate possible duplicates, as some traits may also
+        # appear as normal attributes in the dir() call.
+        words = list(set(words))
+        words.sort()
+
+    # filter out non-string attributes which may be stuffed by dir() calls
+    # and poor coding in third-party modules
+    return [w for w in words if isinstance(w, basestring)]
+
 #----------------------------------------------------------------------------
 def import_fail_info(mod_name,fns=None):
     """Inform load failure for a module."""
