@@ -32,7 +32,7 @@ ip.set_hook('editor', calljed)
 You can then enable the functionality by doing 'import myiphooks'
 somewhere in your configuration files or ipython command line.
 
-$Id: hooks.py 1322 2006-05-24 07:51:39Z fperez $"""
+$Id: hooks.py 2998 2008-01-31 10:06:04Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2005 Fernando Perez. <fperez@colorado.edu>
@@ -48,13 +48,15 @@ __license__ = Release.license
 __version__ = Release.version
 
 import os,bisect
-from genutils import Term
+from genutils import Term,shell
 from pprint import PrettyPrinter
 
 # List here all the default hooks.  For now it's just the editor functions
 # but over time we'll move here all the public API for user-accessible things.
 __all__ = ['editor', 'fix_error_editor', 'result_display',
-           'input_prefilter', 'shutdown_hook', 'late_startup_hook']
+           'input_prefilter', 'shutdown_hook', 'late_startup_hook',
+           'generate_prompt', 'generate_output_prompt','shell_hook',
+           'show_in_pager','pre_prompt_hook', 'pre_runcode_hook']
 
 pformat = PrettyPrinter().pformat
 
@@ -136,6 +138,8 @@ class CommandChainDispatcher:
                 if exc.args or exc.kwargs:
                     args = exc.args
                     kw = exc.kwargs
+        # if no function will accept it, raise TryNext up to the caller
+        raise ipapi.TryNext
                 
     def __str__(self):
         return str(self.chain)
@@ -144,6 +148,13 @@ class CommandChainDispatcher:
         """ Add a func to the cmd chain with given priority """
         bisect.insort(self.chain,(priority,func))
 
+    def __iter__(self):
+        """ Return all objects in chain.
+        
+        Handy if the objects are not callable.
+        """
+        return iter(self.chain)
+    
 def result_display(self,arg):
     """ Default display hook.
     
@@ -194,3 +205,39 @@ def late_startup_hook(self):
     
     """
     #print "default startup hook ok" # dbg
+
+def generate_prompt(self, is_continuation):
+    """ calculate and return a string with the prompt to display """
+    ip = self.api
+    if is_continuation:
+        return str(ip.IP.outputcache.prompt2)
+    return str(ip.IP.outputcache.prompt1)
+
+def generate_output_prompt(self):
+    ip = self.api
+    return str(ip.IP.outputcache.prompt_out)
+
+def shell_hook(self,cmd):
+    """ Run system/shell command a'la os.system() """
+
+    shell(cmd, header=self.rc.system_header, verbose=self.rc.system_verbose)
+
+def show_in_pager(self,s):
+    """ Run a string through pager """
+    # raising TryNext here will use the default paging functionality
+    raise ipapi.TryNext
+
+def pre_prompt_hook(self):
+    """ Run before displaying the next prompt
+    
+    Use this e.g. to display output from asynchronous operations (in order 
+    to not mess up text entry)   
+    """
+    
+    return None
+
+def pre_runcode_hook(self):
+    """ Executed before running the (prefiltered) code in IPython """
+    return None
+    
+
