@@ -64,8 +64,14 @@ print "done!"
 import __builtin__
 import sys
 
+try: # Python 2.3 compatibility
+    set
+except NameError:
+    import sets
+    set = sets.Set
+
 # our own
-from IPython.genutils import warn,error
+#from IPython.genutils import warn,error
  
 class TryNext(Exception):
     """Try next hook exception.
@@ -80,6 +86,29 @@ class TryNext(Exception):
         self.args = args
         self.kwargs = kwargs
 
+class UsageError(Exception):
+    """ Error in magic function arguments, etc.
+    
+    Something that probably won't warrant a full traceback, but should
+    nevertheless interrupt a macro / batch file.   
+    """
+
+class IPyAutocall:
+    """ Instances of this class are always autocalled
+    
+    This happens regardless of 'autocall' variable state. Use this to
+    develop macro-like mechanisms.
+    """
+    
+    def set_ip(self,ip):
+        """ Will be used to set _ip point to current ipython instance b/f call
+        
+        Override this method if you don't want this to happen.
+        
+        """
+        self._ip = ip
+    
+
 # contains the most recently instantiated IPApi
 
 class IPythonNotRunning:
@@ -109,7 +138,7 @@ class IPythonNotRunning:
     def _dummy_warn(self,*args,**kw):
         """Dummy function, which doesn't do anything but warn."""
 
-        warn("IPython is not running, this is a dummy no-op function")
+        print ("IPython is not running, this is a dummy no-op function")
 
     def _dummy_silent(self,*args,**kw):
         """Dummy function, which doesn't do anything and emits no warnings."""
@@ -150,13 +179,14 @@ class IPApi:
 
         self.magic = ip.ipmagic
         
-        self.system = ip.ipsystem
+        self.system = ip.system
         
         self.set_hook = ip.set_hook
         
         self.set_custom_exc = ip.set_custom_exc
 
         self.user_ns = ip.user_ns
+        self.user_ns['_ip'] = self
 
         self.set_crash_handler = ip.set_crash_handler
 
@@ -167,6 +197,10 @@ class IPApi:
         # The ipython instance provided
         self.IP = ip
 
+        self.extensions = {}
+
+        self.dbg = DebugTools(self)
+        
         global _recent
         _recent = self
 
@@ -205,6 +239,11 @@ class IPApi:
                 
         import new
         im = new.instancemethod(func,self.IP, self.IP.__class__)
+        old = getattr(self.IP, "magic_" + magicname, None)
+        if old:
+            self.dbg.debug_stack("Magic redefinition '%s', old %s" % (magicname,
+                                 old))
+            
         setattr(self.IP, "magic_" + magicname, im)
     
     def ex(self,cmd):
@@ -224,18 +263,59 @@ class IPApi:
         
         Takes either all lines in one string or list of lines.
         """
-        if isinstance(lines,basestring):
-            self.IP.runlines(lines)
-        else:
-            self.IP.runlines('\n'.join(lines))
 
-    def to_user_ns(self,vars):
+        def cleanup_ipy_script(script):
+            """ Make a script safe for _ip.runlines() 
+            
+            - Removes empty lines
+            - Suffixes all indented blocks that end with unindented lines with empty lines
+            
+            """
+            res = []
+            lines = script.splitlines()
+
+            level = 0
+            for l in lines:
+                lstripped = l.lstrip()
+                stripped = l.strip()                
+                if not stripped:
+                    continue
+                newlevel = len(l) - len(lstripped)
+                def is_secondary_block_start(s):
+                    if not s.endswith(':'):
+                        return False
+                    if (s.startswith('elif') or 
+                        s.startswith('else') or 
+                        s.startswith('except') or
+                        s.startswith('finally')):
+                        return True
+                        
+                if level > 0 and newlevel == 0 and not is_secondary_block_start(stripped): 
+                    # add empty line
+                    res.append('')
+                    
+                res.append(l)
+                level = newlevel
+            return '\n'.join(res) + '\n'
+        
+        if isinstance(lines,basestring):
+            script = lines            
+        else:
+            script = '\n'.join(lines)
+        clean=cleanup_ipy_script(script)
+        # print "_ip.runlines() script:\n",clean #dbg
+        self.IP.runlines(clean)
+    def to_user_ns(self,vars, interactive = True):
         """Inject a group of variables into the IPython user namespace.
 
         Inputs:
 
-         - vars: string with variable names separated by whitespace
+         - vars: string with variable names separated by whitespace, or a
+         dict with name/value pairs.
 
+         - interactive: if True (default), the var will be listed with
+        %whos et. al.
+         
         This utility routine is meant to ease interactive debugging work,
         where you want to easily propagate some internal variable in your code
         up to the interactive namespace for further exploration.
@@ -264,29 +344,54 @@ class IPApi:
             # This pushes x and y to the interactive prompt immediately, even
             # if this routine crashes on the next line after:
             ip.to_user_ns('x y')
+            ...
+            
+            # To expose *ALL* the local variables from the function, use:
+            ip.to_user_ns(locals())
+
             ...
             # return           
         
-        If you need to rename variables, just use ip.user_ns with dict
-        and update:
-        
-        # exposes variables 'foo' as 'x' and 'bar' as 'y' in IPython 
-        # user namespace
-        ip.user_ns.update(dict(x=foo,y=bar))    
+
+        If you need to rename variables, the dict input makes it easy.  For
+        example, this call exposes variables 'foo' as 'x' and 'bar' as 'y'
+        in IPython user namespace:
+
+        ip.to_user_ns(dict(x=foo,y=bar))    
         """
 
         # print 'vars given:',vars # dbg
-        # Get the caller's frame to evaluate the given names in
-        cf = sys._getframe(1)
         
-        user_ns = self.user_ns
-        
-        for name in vars.split():
-            try:
-                user_ns[name] = eval(name,cf.f_globals,cf.f_locals)
-            except:
-                error('could not get var. %s from %s' %
-                (name,cf.f_code.co_name))
+        # We need a dict of name/value pairs to do namespace updates.
+        if isinstance(vars,dict):
+            # If a dict was given, no need to change anything.
+            vdict = vars
+        elif isinstance(vars,basestring):
+            # If a string with names was given, get the caller's frame to
+            # evaluate the given names in
+            cf = sys._getframe(1)
+            vdict = {}
+            for name in vars.split():
+                try:
+                    vdict[name] = eval(name,cf.f_globals,cf.f_locals)
+                except:
+                    print ('could not get var. %s from %s' %
+                    (name,cf.f_code.co_name))
+        else:
+            raise ValueError('vars must be a string or a dict')
+            
+        # Propagate variables to user namespace
+        self.user_ns.update(vdict)
+
+        # And configure interactive visibility
+        config_ns = self.IP.user_config_ns
+        if interactive:
+            for name,val in vdict.iteritems():
+                config_ns.pop(name,None)
+        else:
+            for name,val in vdict.iteritems():
+                config_ns[name] = val                    
+
 
     def expand_alias(self,line):
         """ Expand an alias in the command line 
@@ -302,6 +407,14 @@ class IPApi:
         res = pre + self.IP.expand_aliases(fn,rest)
         return res
 
+    def itpl(self, s, depth = 1):
+        """ Expand Itpl format string s.
+        
+        Only callable from command line (i.e. prefilter results);
+        If you use in your scripts, you need to use a bigger depth!
+        """
+        return self.IP.var_expand(s, depth)
+        
     def defalias(self, name, cmd):
         """ Define a new alias
         
@@ -309,15 +422,32 @@ class IPApi:
         
         Creates a new alias named 'bb' in ipython user namespace
         """
+
+        self.dbg.check_hotname(name)
         
-        
-        nargs = cmd.count('%s')
-        if nargs>0 and cmd.find('%l')>=0:
-            raise Exception('The %s and %l specifiers are mutually exclusive '
-                            'in alias definitions.')
+
+        if name in self.IP.alias_table:
+            self.dbg.debug_stack("Alias redefinition: '%s' => '%s' (old '%s')" %
+                             (name, cmd, self.IP.alias_table[name]))
+            
+
+        if callable(cmd):
+            self.IP.alias_table[name] = cmd
+            import IPython.shadowns
+            setattr(IPython.shadowns, name,cmd)
+            return
+            
+        if isinstance(cmd,basestring):
+            nargs = cmd.count('%s')
+            if nargs>0 and cmd.find('%l')>=0:
+                raise Exception('The %s and %l specifiers are mutually exclusive '
+                                'in alias definitions.')
                   
-        else:  # all looks OK
-            self.IP.alias_table[name] = (nargs,cmd)            
+            self.IP.alias_table[name] = (nargs,cmd)
+            return
+        
+        # just put it in - it's probably (0,'foo')
+        self.IP.alias_table[name] = cmd
     
     def defmacro(self, *args):
         """ Define a new macro
@@ -342,9 +472,80 @@ class IPApi:
         else:
             return Exception("_ip.defmacro must be called with 1 or 2 arguments")
         
+    def set_next_input(self, s):
+        """ Sets the 'default' input string for the next command line.
         
+        Requires readline.
+        
+        Example:
+        
+        [D:\ipython]|1> _ip.set_next_input("Hello Word")
+        [D:\ipython]|2> Hello Word_  # cursor is here        
+        """
 
-def launch_new_instance(user_ns = None):
+        self.IP.rl_next_input = s
+
+    def load(self, mod):
+        """ Load an extension.
+        
+        Some modules should (or must) be 'load()':ed, rather than just imported.
+        
+        Loading will do:
+        
+        - run init_ipython(ip)
+        - run ipython_firstrun(ip)
+        
+        """
+        if mod in self.extensions:
+            # just to make sure we don't init it twice
+            # note that if you 'load' a module that has already been
+            # imported, init_ipython gets run anyway
+            
+            return self.extensions[mod]
+        __import__(mod)
+        m = sys.modules[mod]
+        if hasattr(m,'init_ipython'):
+            m.init_ipython(self)
+            
+        if hasattr(m,'ipython_firstrun'):
+            already_loaded = self.db.get('firstrun_done', set())
+            if mod not in already_loaded:
+                m.ipython_firstrun(self)
+                already_loaded.add(mod)
+                self.db['firstrun_done'] = already_loaded
+            
+        self.extensions[mod] = m
+        return m
+
+
+class DebugTools:
+    """ Used for debugging mishaps in api usage
+    
+    So far, tracing redefinitions is supported.
+    """
+    
+    def __init__(self, ip):
+        self.ip = ip
+        self.debugmode = False
+        self.hotnames = set()
+        
+    def hotname(self, name_to_catch):
+        self.hotnames.add(name_to_catch)
+        
+    def debug_stack(self, msg = None):
+        if not self.debugmode:
+            return
+        
+        import traceback
+        if msg is not None:
+            print '====== %s  ========' % msg
+        traceback.print_stack()
+
+    def check_hotname(self,name):
+        if name in self.hotnames:
+            self.debug_stack( "HotName '%s' caught" % name)
+
+def launch_new_instance(user_ns = None,shellclass = None):
     """ Make and start a new ipython instance.
     
     This can be called even without having an already initialized 
@@ -353,7 +554,7 @@ def launch_new_instance(user_ns = None):
     This is also used as the egg entry point for the 'ipython' script.
     
     """
-    ses = make_session(user_ns)
+    ses = make_session(user_ns,shellclass)
     ses.mainloop()
 
 
@@ -389,7 +590,7 @@ def make_user_global_ns(ns = None):
     return ns
 
 
-def make_session(user_ns = None):
+def make_session(user_ns = None, shellclass = None):
     """Makes, but does not launch an IPython session.
     
     Later on you can call obj.mainloop() on the returned object.
@@ -401,6 +602,7 @@ def make_session(user_ns = None):
     
     WARNING: This should *not* be run when a session exists already."""
 
-    import IPython
-    return IPython.Shell.start(user_ns)
-
+    import IPython.Shell
+    if shellclass is None:
+        return IPython.Shell.start(user_ns)
+    return shellclass(user_ns = user_ns)
