@@ -6,7 +6,7 @@ Requires Python 2.1 or better.
 
 This file contains the main make_IPython() starter function.
 
-$Id: ipmaker.py 2156 2007-03-19 02:32:19Z fperez $"""
+$Id: ipmaker.py 2930 2008-01-11 07:03:11Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -58,6 +58,14 @@ from IPython.iplib import InteractiveShell
 from IPython.usage import cmd_line_usage,interactive_usage
 from IPython.genutils import *
 
+def force_import(modname):
+    if modname in sys.modules:
+        print "reload",modname
+        reload(sys.modules[modname])
+    else:
+        __import__(modname)
+        
+
 #-----------------------------------------------------------------------------
 def make_IPython(argv=None,user_ns=None,user_global_ns=None,debug=1,
                  rc_override=None,shell_class=InteractiveShell,
@@ -95,8 +103,12 @@ def make_IPython(argv=None,user_ns=None,user_global_ns=None,debug=1,
                      embedded=embedded,**kw)
 
     # Put 'help' in the user namespace
-    from site import _Helper
-    IP.user_ns['help'] = _Helper()
+    try:
+        from site import _Helper
+        IP.user_ns['help'] = _Helper()
+    except ImportError:
+        warn('help() not available - check site.py')
+    IP.user_config_ns = {}
 
 
     if DEVDEBUG:
@@ -110,10 +122,11 @@ def make_IPython(argv=None,user_ns=None,user_global_ns=None,debug=1,
                          % (sys.version.split('\n')[0],),
                          "IPython %s -- An enhanced Interactive Python."
                          % (__version__,),
-"""?       -> Introduction to IPython's features.
-%magic  -> Information about IPython's 'magic' % functions.
-help    -> Python's own help system.
-object? -> Details about 'object'. ?object also works, ?? prints more.
+"""\
+?         -> Introduction and overview of IPython's features.
+%quickref -> Quick reference.
+help      -> Python's own help system.
+object?   -> Details about 'object'. ?object also works, ?? prints more.
 """ ]
 
     IP.usage = interactive_usage
@@ -159,6 +172,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                     'debug! deep_reload! editor=s log|l messages! nosep '
                     'object_info_string_level=i pdb! '
                     'pprint! prompt_in1|pi1=s prompt_in2|pi2=s prompt_out|po=s '
+                    'pydb! '
                     'pylab_import_all! '
                     'quick screen_length|sl=i prompts_pad_left=i '
                     'logfile|lf=s logplay|lp=s profile|p=s '
@@ -173,10 +187,10 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
 
     # Options that can *only* appear at the cmd line (not in rcfiles).
     
-    # The "ignore" option is a kludge so that Emacs buffers don't crash, since
-    # the 'C-c !' command in emacs automatically appends a -i option at the end.
-    cmdline_only = ('help ignore|i ipythondir=s Version upgrade '
-                    'gthread! qthread! q4thread! wthread! pylab! tk!')
+    cmdline_only = ('help interact|i ipythondir=s Version upgrade '
+                    'gthread! qthread! q4thread! wthread! tkthread! pylab! tk! '
+                    # 'twisted!'  # disabled for now.
+                    )
 
     # Build the actual name list to be used by DPyGetOpt
     opts_names = qw(cmdline_opts) + qw(cmdline_only)
@@ -187,26 +201,28 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       autoedit_syntax = 0,
                       autoindent = 0,
                       automagic = 1,
+                      autoexec = [],
                       banner = 1,
-                      cache_size = 1000,
                       c = '',
+                      cache_size = 1000,
                       classic = 0,
-                      colors = 'NoColor',
                       color_info = 0,
+                      colors = 'NoColor',
                       confirm_exit = 1,
                       debug = 0,
                       deep_reload = 0,
                       editor = '0',
+                      gthread = 0,
                       help = 0,
-                      ignore = 0,
+                      interact = 0,
                       ipythondir = ipythondir_def,
                       log = 0,
                       logfile = '',
                       logplay = '',
-                      multi_line_specials = 1,
                       messages = 1,
-                      object_info_string_level = 0,
+                      multi_line_specials = 1,
                       nosep = 0,
+                      object_info_string_level = 0,
                       pdb = 0,
                       pprint = 0,
                       profile = '',
@@ -214,31 +230,31 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       prompt_in2 = '   .\\D.: ',
                       prompt_out = 'Out[\\#]: ',
                       prompts_pad_left = 1,
+                      pylab = 0,
                       pylab_import_all = 1,
-                      quiet = 0,
+                      q4thread = 0,
+                      qthread = 0,
                       quick = 0,
+                      quiet = 0,
+                      rcfile = 'ipythonrc' + rc_suffix,
                       readline = 1,
                       readline_merge_completions = 1,
                       readline_omit__names = 0,
-                      rcfile = 'ipythonrc' + rc_suffix,
                       screen_length = 0,
                       separate_in = '\n',
                       separate_out = '\n',
                       separate_out2 = '',
                       system_header = 'IPython system call: ',
                       system_verbose = 0,
-                      gthread = 0,
-                      qthread = 0,
-                      q4thread = 0,
-                      wthread = 0,
-                      pylab = 0,
                       term_title = 1,
                       tk = 0,
+                      #twisted= 0,  # disabled for now
                       upgrade = 0,
                       Version = 0,
-                      xmode = 'Verbose',
                       wildcards_case_sensitive = 1,
+                      wthread = 0,
                       wxversion = '0',
+                      xmode = 'Context',
                       magic_docstrings = 0,  # undocumented, for doc generation
                       )
     
@@ -299,9 +315,9 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
 
     try:
         getopt.processArguments(argv)
-    except:
+    except DPyGetOpt.ArgumentError, exc:
         print cmd_line_usage
-        warn('\nError in Arguments: ' + `sys.exc_value`)
+        warn('\nError in Arguments: "%s"' % exc)
         sys.exit(1)
 
     # convert the options dict to a struct for much lighter syntax later
@@ -423,12 +439,14 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
              % (opts_all.rcfile) )
 
     # 'profiles' are a shorthand notation for config filenames
+    profile_handled_by_legacy = False
     if opts_all.profile:
             
         try:
             opts_all.rcfile = filefind('ipythonrc-' + opts_all.profile
                                        + rc_suffix,
                                        opts_all.ipythondir)
+            profile_handled_by_legacy = True
         except IOError:
            if opts_all.debug:  IP.InteractiveTB()
            opts.profile = ''  # remove profile from options if invalid
@@ -623,32 +641,32 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
         IP.InteractiveTB()
         import_fail_info('ipy_system_conf')
         
-    if opts_all.profile:
+    # only import prof module if ipythonrc-PROF was not found
+    if opts_all.profile and not profile_handled_by_legacy:
         profmodname = 'ipy_profile_' + opts_all.profile
         try:
-            __import__(profmodname)
-        except ImportError:
-            # only warn if ipythonrc-PROFNAME didn't exist
-            if opts.profile =='':
-                warn("Could not start with profile '%s'!\n"
-                     "('%s/%s.py' does not exist? run '%%upgrade')" %
-                     (opts_all.profile, opts_all.ipythondir, profmodname) )
-        except:            
-            print "Error importing",profmodname,"- perhaps you should run %upgrade?"
+            
+            force_import(profmodname)
+        except:
             IP.InteractiveTB()
+            print "Error importing",profmodname,"- perhaps you should run %upgrade?"
             import_fail_info(profmodname)
-                      
-    try:    
-        import ipy_user_conf
-    except ImportError:
-        if opts_all.debug:  IP.InteractiveTB()
-        warn("Could not import user config!\n "
-             "('%s/ipy_user_conf.py' does not exist? Please run '%%upgrade')\n"
-             % opts_all.ipythondir)
+    else:
+        force_import('ipy_profile_none')
+    try:
+            
+        force_import('ipy_user_conf')
+        
     except:
-        print "Error importing ipy_user_conf - perhaps you should run %upgrade?"
+        conf = opts_all.ipythondir + "/ipy_user_conf.py"
         IP.InteractiveTB()
+        if not os.path.isfile(conf):
+            warn(conf + ' does not exist, please run %upgrade!')
+
         import_fail_info("ipy_user_conf")
+
+    # finally, push the argv to options again to ensure highest priority
+    IP_rc.update(opts)
     
     # release stdout and stderr and save config log into a global summary
     msg.config.release_all()
@@ -664,7 +682,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # Take a snapshot of the user namespace before opening the shell. That way
     # we'll be able to identify which things were interactively defined and
     # which were defined through config files.
-    IP.user_config_ns = IP.user_ns.copy()
+    IP.user_config_ns.update(IP.user_ns)
 
     # Force reading a file as if it were a session log. Slower but safer.
     if load_logplay:
