@@ -23,14 +23,34 @@ __license__ = Release.license
 import os
 
 ignore_termtitle = 0
+
+try:
+    import ctypes
+    SetConsoleTitleW=ctypes.windll.kernel32.SetConsoleTitleW
+    SetConsoleTitleW.argtypes=[ctypes.c_wchar_p]
+    def _set_term_title(title):
+        """ Set terminal title using the ctypes"""
+        SetConsoleTitleW(title)
+
+except ImportError:
+    def _set_term_title(title):
+        """ Set terminal title using the 'title' command """
+        curr=os.getcwd()
+        os.chdir("C:") #Cannot be on network share when issuing system commands
+        ret = os.system("title " + title)
+        os.chdir(curr)
+        if ret:
+            ignore_termtitle = 1
+
 def set_term_title(title):
     """ Set terminal title using the 'title' command """
-    
     global ignore_termtitle
-    
-    if not ignore_termtitle:
-        ret = os.system("title " + title)
-    if ret:
-        ignore_termtitle = 1
-    
-    
+
+    if ignore_termtitle:
+        return
+    _set_term_title(title)
+
+def freeze_term_title():
+    global ignore_termtitle
+    ignore_termtitle = 1
+
