@@ -60,7 +60,7 @@ You can implement other color schemes easily, the syntax is fairly
 self-explanatory. Please send back new schemes you develop to the author for
 possible inclusion in future releases.
 
-$Id: ultraTB.py 1154 2006-02-11 23:20:05Z fperez $"""
+$Id: ultraTB.py 2908 2007-12-30 21:07:46Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001 Nathaniel Gray <n8gray@caltech.edu>
@@ -81,6 +81,7 @@ import keyword
 import linecache
 import os
 import pydoc
+import re
 import string
 import sys
 import time
@@ -88,9 +89,14 @@ import tokenize
 import traceback
 import types
 
+# For purposes of monkeypatching inspect to fix a bug in it.
+from inspect import getsourcefile, getfile, getmodule,\
+     ismodule,  isclass, ismethod, isfunction, istraceback, isframe, iscode
+
+
 # IPython's own modules
 # Modified pdb which doesn't damage IPython's readline handling
-from IPython import Debugger
+from IPython import Debugger, PyColorize
 from IPython.ipstruct import Struct
 from IPython.excolors import ExceptionColors
 from IPython.genutils import Term,uniq_stable,error,info
@@ -99,6 +105,12 @@ from IPython.genutils import Term,uniq_stable,error,info
 # amount of space to put line numbers before verbose tracebacks
 INDENT_SIZE = 8
 
+# Default color scheme.  This is used, for example, by the traceback
+# formatter.  When running in an actual IPython instance, the user's rc.colors
+# value is used, but havinga module global makes this functionality available
+# to users of ultraTB who are NOT running inside ipython.
+DEFAULT_SCHEME = 'NoColor'
+
 #---------------------------------------------------------------------------
 # Code begins
 
@@ -111,11 +123,111 @@ def inspect_error():
     error('Internal Python error in the inspect module.\n'
           'Below is the traceback from this internal error.\n')
 
+
+def findsource(object):
+    """Return the entire source file and starting line number for an object.
+
+    The argument may be a module, class, method, function, traceback, frame,
+    or code object.  The source code is returned as a list of all the lines
+    in the file and the line number indexes a line in that list.  An IOError
+    is raised if the source code cannot be retrieved.
+
+    FIXED version with which we monkeypatch the stdlib to work around a bug."""
+
+    file = getsourcefile(object) or getfile(object)
+    # If the object is a frame, then trying to get the globals dict from its
+    # module won't work. Instead, the frame object itself has the globals
+    # dictionary.
+    globals_dict = None
+    if inspect.isframe(object):
+        # XXX: can this ever be false?
+        globals_dict = object.f_globals
+    else:
+        module = getmodule(object, file)
+        if module:
+            globals_dict = module.__dict__
+    lines = linecache.getlines(file, globals_dict)
+    if not lines:
+        raise IOError('could not get source code')
+
+    if ismodule(object):
+        return lines, 0
+
+    if isclass(object):
+        name = object.__name__
+        pat = re.compile(r'^(\s*)class\s*' + name + r'\b')
+        # make some effort to find the best matching class definition:
+        # use the one with the least indentation, which is the one
+        # that's most probably not inside a function definition.
+        candidates = []
+        for i in range(len(lines)):
+            match = pat.match(lines[i])
+            if match:
+                # if it's at toplevel, it's already the best one
+                if lines[i][0] == 'c':
+                    return lines, i
+                # else add whitespace to candidate list
+                candidates.append((match.group(1), i))
+        if candidates:
+            # this will sort by whitespace, and by line number,
+            # less whitespace first
+            candidates.sort()
+            return lines, candidates[0][1]
+        else:
+            raise IOError('could not find class definition')
+
+    if ismethod(object):
+        object = object.im_func
+    if isfunction(object):
+        object = object.func_code
+    if istraceback(object):
+        object = object.tb_frame
+    if isframe(object):
+        object = object.f_code
+    if iscode(object):
+        if not hasattr(object, 'co_firstlineno'):
+            raise IOError('could not find function definition')
+        pat = re.compile(r'^(\s*def\s)|(.*(?<!\w)lambda(:|\s))|^(\s*@)')
+        pmatch = pat.match
+        # fperez - fix: sometimes, co_firstlineno can give a number larger than
+        # the length of lines, which causes an error.  Safeguard against that.
+        lnum = min(object.co_firstlineno,len(lines))-1
+        while lnum > 0:
+            if pmatch(lines[lnum]): break
+            lnum -= 1
+ 
+        return lines, lnum
+    raise IOError('could not find code object')
+
+# Monkeypatch inspect to apply our bugfix.  This code only works with py25
+if sys.version_info[:2] >= (2,5):
+    inspect.findsource = findsource
+
+def fix_frame_records_filenames(records):
+    """Try to fix the filenames in each record from inspect.getinnerframes().
+
+    Particularly, modules loaded from within zip files have useless filenames
+    attached to their code object, and inspect.getinnerframes() just uses it.
+    """
+    fixed_records = []
+    for frame, filename, line_no, func_name, lines, index in records:
+        # Look inside the frame's globals dictionary for __file__, which should
+        # be better.
+        better_fn = frame.f_globals.get('__file__', None)
+        if isinstance(better_fn, str):
+            # Check the type just in case someone did something weird with
+            # __file__. It might also be None if the error occurred during
+            # import.
+            filename = better_fn
+        fixed_records.append((frame, filename, line_no, func_name, lines, index))
+    return fixed_records
+
+
 def _fixed_getinnerframes(etb, context=1,tb_offset=0):
     import linecache
     LNUM_POS, LINES_POS, INDEX_POS =  2, 4, 5
 
-    records  = inspect.getinnerframes(etb, context)
+    records  = fix_frame_records_filenames(inspect.getinnerframes(etb, context))
 
     # If the error is at the console, don't build any context, since it would
     # otherwise produce 5 blank lines printed out (there is no file at the
@@ -151,11 +263,26 @@ def _fixed_getinnerframes(etb, context=1,tb_offset=0):
 # functionality to produce a pseudo verbose TB for SyntaxErrors, so that they
 # can be recognized properly by ipython.el's py-traceback-line-re
 # (SyntaxErrors have to be treated specially because they have no traceback)
-def _formatTracebackLines(lnum, index, lines, Colors, lvals=None):
+
+_parser = PyColorize.Parser()
+    
+def _formatTracebackLines(lnum, index, lines, Colors, lvals=None,scheme=None):
     numbers_width = INDENT_SIZE - 1
     res = []
     i = lnum - index
+
+    # This lets us get fully syntax-highlighted tracebacks.
+    if scheme is None:
+        try:
+            scheme = __IPYTHON__.rc.colors
+        except:
+            scheme = DEFAULT_SCHEME
+    _line_format = _parser.format2
+
     for line in lines:
+        new_line, err = _line_format(line,'str',scheme)
+        if not err: line = new_line
+        
         if i == lnum:
             # This is the line with the error
             pad = numbers_width - len(str(i))
@@ -181,6 +308,7 @@ def _formatTracebackLines(lnum, index, lines, Colors, lvals=None):
         i = i + 1
     return res
 
+
 #---------------------------------------------------------------------------
 # Module classes
 class TBTools:
@@ -204,10 +332,14 @@ class TBTools:
 
     def set_colors(self,*args,**kw):
         """Shorthand access to the color table scheme selector method."""
-        
+
+        # Set own color table
         self.color_scheme_table.set_active_scheme(*args,**kw)
         # for convenience, set Colors to the active scheme
         self.Colors = self.color_scheme_table.active_colors
+        # Also set colors of debugger
+        if hasattr(self,'pdb') and self.pdb is not None:
+            self.pdb.set_colors(*args,**kw)
 
     def color_toggle(self):
         """Toggle between the currently active color scheme and NoColor."""
@@ -244,7 +376,9 @@ class ListTB(TBTools):
         TBTools.__init__(self,color_scheme = color_scheme,call_pdb=0)
         
     def __call__(self, etype, value, elist):
+        Term.cout.flush()
         print >> Term.cerr, self.text(etype,value,elist)
+        Term.cerr.flush()
 
     def text(self,etype, value, elist,context=5):
         """Return a color formatted string with the traceback info."""
@@ -314,9 +448,9 @@ class ListTB(TBTools):
         
         Colors = self.Colors
         list = []
-        if type(etype) == types.ClassType:
+        try:
             stype = Colors.excName + etype.__name__ + Colors.Normal
-        else:
+        except AttributeError:
             stype = etype  # String exceptions don't get special coloring
         if value is None:
             list.append( str(stype) + '\n')
@@ -391,12 +525,17 @@ class VerboseTB(TBTools):
         """Return a nice text document describing the traceback."""
 
         # some locals
+        try:
+            etype = etype.__name__
+        except AttributeError:
+            pass
         Colors        = self.Colors   # just a shorthand + quicker name lookup
         ColorsNormal  = Colors.Normal  # used a lot
+        col_scheme    = self.color_scheme_table.active_scheme_name
         indent        = ' '*INDENT_SIZE
-        exc           = '%s%s%s' % (Colors.excName, str(etype), ColorsNormal)
         em_normal     = '%s\n%s%s' % (Colors.valEm, indent,ColorsNormal)
         undefined     = '%sundefined%s' % (Colors.em, ColorsNormal)
+        exc = '%s%s%s' % (Colors.excName,etype,ColorsNormal)
 
         # some internal-use functions
         def text_repr(value):
@@ -430,8 +569,10 @@ class VerboseTB(TBTools):
         def nullrepr(value, repr=text_repr): return ''
 
         # meat of the code begins
-        if type(etype) is types.ClassType:
+        try:
             etype = etype.__name__
+        except AttributeError:
+            pass
 
         if self.long_header:
             # Header with the exception type, python version, and date
@@ -631,7 +772,8 @@ class VerboseTB(TBTools):
                 frames.append(level)
             else:
                 frames.append('%s%s' % (level,''.join(
-                    _formatTracebackLines(lnum,index,lines,self.Colors,lvals))))
+                    _formatTracebackLines(lnum,index,lines,Colors,lvals,
+                                          col_scheme))))
 
         # Get (safely) a string form of the exception info
         try:
@@ -662,8 +804,16 @@ class VerboseTB(TBTools):
         # return all our info assembled as a single string
         return '%s\n\n%s\n%s' % (head,'\n'.join(frames),''.join(exception[0]) )
 
-    def debugger(self):
-        """Call up the pdb debugger if desired, always clean up the tb reference.
+    def debugger(self,force=False):
+        """Call up the pdb debugger if desired, always clean up the tb
+        reference.
+
+        Keywords:
+
+          - force(False): by default, this routine checks the instance call_pdb
+          flag and does not actually invoke the debugger if the flag is false.
+          The 'force' option forces the debugger to activate even if the flag
+          is false.
 
         If the call_pdb flag is set, the pdb interactive debugger is
         invoked. In all cases, the self.tb reference to the current traceback
@@ -674,7 +824,7 @@ class VerboseTB(TBTools):
         requires a special setup for the readline completers, you'll have to
         fix that by hand after invoking the exception handler."""
 
-        if self.call_pdb:
+        if force or self.call_pdb:
             if self.pdb is None:
                 self.pdb = Debugger.Pdb(
                     self.color_scheme_table.active_scheme_name)
@@ -684,7 +834,10 @@ class VerboseTB(TBTools):
             sys.displayhook = sys.__displayhook__
             self.pdb.reset()
             # Find the right frame so we don't pop up inside ipython itself
-            etb = self.tb
+            if hasattr(self,'tb'):
+                etb = self.tb
+            else:
+                etb = self.tb = sys.last_traceback
             while self.tb.tb_next is not None:
                 self.tb = self.tb.tb_next
             try:
@@ -692,17 +845,18 @@ class VerboseTB(TBTools):
                     etb = etb.tb_next
                 self.pdb.botframe = etb.tb_frame
                 self.pdb.interaction(self.tb.tb_frame, self.tb)
-            except:
-                print '*** ERROR ***'
-                print 'This version of pdb has a bug and crashed.'
-                print 'Returning to IPython...'
-            sys.displayhook = dhook
-        del self.tb
+            finally:
+                sys.displayhook = dhook
+            
+        if hasattr(self,'tb'):
+            del self.tb
 
     def handler(self, info=None):
         (etype, evalue, etb) = info or sys.exc_info()
         self.tb = etb
+        Term.cout.flush()
         print >> Term.cerr, self.text(etype, evalue, etb)
+        Term.cerr.flush()
 
     # Changed so an instance can just be called as VerboseTB_inst() and print
     # out the right info on its own.
@@ -712,7 +866,10 @@ class VerboseTB(TBTools):
             self.handler()
         else:
             self.handler((etype, evalue, etb))
-        self.debugger()
+        try:
+            self.debugger()
+        except KeyboardInterrupt:
+            print "\nKeyboardInterrupt"
 
 #----------------------------------------------------------------------------
 class FormattedTB(VerboseTB,ListTB):
@@ -819,13 +976,18 @@ class AutoFormattedTB(FormattedTB):
         
         if out is None:
             out = Term.cerr
+        Term.cout.flush()
         if tb_offset is not None:
             tb_offset, self.tb_offset = self.tb_offset, tb_offset
             print >> out, self.text(etype, evalue, etb)
             self.tb_offset = tb_offset
         else:
             print >> out, self.text(etype, evalue, etb)
-        self.debugger()
+        out.flush()
+        try:
+            self.debugger()
+        except KeyboardInterrupt:
+            print "\nKeyboardInterrupt"
 
     def text(self,etype=None,value=None,tb=None,context=5,mode=None):
         if etype is None:
