@@ -6,7 +6,7 @@
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-# $Id: usage.py 2152 2007-03-18 20:13:35Z fperez $
+# $Id: usage.py 2723 2007-09-07 07:44:16Z fperez $
 
 from IPython import Release
 __author__  = '%s <%s>' % Release.authors['Fernando']
@@ -602,14 +602,21 @@ quick_reference = r"""
 IPython -- An enhanced Interactive Python - Quick Reference Card
 ================================================================
 
-obj?, obj??, ?obj,??obj   : Get help, or more help for object
-?os.p*           : List names in os starting with p
+obj?, obj??      : Get help, or more help for object (also works as
+                   ?obj, ??obj).
+?foo.*abc*       : List names in 'foo' containing 'abc' in them.
+%magic           : Information about IPython's 'magic' % functions.
+
+Magic functions are prefixed by %, and typically take their arguments without
+parentheses, quotes or even commas for convenience.
+ 
+Example magic function calls:
 
-Example magic:
-    
 %alias d ls -F   : 'd' is now an alias for 'ls -F'
 alias d ls -F    : Works if 'alias' not a python name
 alist = %alias   : Get list of aliases to 'alist'
+cd /usr/share    : Obvious. cd -<tab> to choose from visited dirs.
+%cd??            : See help AND source for magic %cd
 
 System commands:
     
@@ -618,17 +625,17 @@ cp a.txt b/      : after %rehashx, most system commands work without !
 cp ${f}.txt $bar : Variable expansion in magics and system commands
 files = !ls /usr : Capture sytem command output
 files.s, files.l, files.n: "a b c", ['a','b','c'], 'a\nb\nc'
-cd /usr/share    : Obvious, also 'cd d:\home\_ipython' works
 
 History:
 
 _i, _ii, _iii    : Previous, next previous, next next previous input
 _i4, _ih[2:5]    : Input history line 4, lines 2-4
 exec _i81        : Execute input history line #81 again
+%rep 81          : Edit input history line #81        
 _, __, ___       : previous, next previous, next next previous output
 _dh              : Directory history
 _oh              : Output history
-%hist            : Command history
+%hist            : Command history. '%hist -g foo' search history for 'foo'
 
 Autocall:
 
@@ -637,6 +644,11 @@ f 1,2            : f(1,2)
 ,f 1 2           : f("1","2")
 ;f 1 2           : f("1 2")
 
+Remember: TAB completion works in many contexts, not just file names
+or python names.
+
+The following magic functions are currently available:
+
 """
 
 
